/*******************************************************************************
*                                 Sudoku                                       *
********************************************************************************
*            Copyright (C) 2006 Sander Jansen. All Rights Reserved             *
*                               ---                                            *
* This program is free software; you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation; either version 2 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA    *
*******************************************************************************/
/*
$Id$
*/
#include <limits.h>
#include "fx.h"
#include "dlx.h"

/*
  Dancing Links Implementation as described by Donald E. Knuth
  http://www-cs-faculty.stanford.edu/~knuth/preprints.html (P159)
*/

dlxnode::dlxnode() : left(NULL),right(NULL),up(NULL),down(NULL),column(NULL) {
  }

dlxcolumn::dlxcolumn() : dlxnode(), size(0), name(0) {
  up=down=this;
  }

dlx::dlx(int num_columns,int num_nodes,int num_solutions) : root(NULL),columns(NULL),nodes(NULL),nindex(0),found(0),solution(NULL) {
  register int i=0;

  columns=new dlxcolumn[num_columns];
  nodes=new dlxnode[num_nodes];
  root=new dlxnode;
  solution=new dlxnode*[num_solutions];

  for (i=0;i<num_solutions;i++)
    solution[i]=NULL;

  columns[0].name=1;
  columns[0].right=NULL;
  for (i=1;i<num_columns;i++) {
    columns[i].name=(i+1);
    columns[i].left=&columns[i-1];
    columns[i-1].right=&columns[i];
    }

  columns[0].left = root;
  columns[num_columns-1].right = root;
  root->right = &columns[0];
  root->left  = &columns[num_columns-1];
  }

dlx::~dlx(){
  delete [] columns;
  delete [] nodes;
  delete [] solution;
  delete root;
  }

void dlx::uncover(dlxcolumn * column){
  dlxnode  * col = column->up;
  while(col!=column){
    dlxnode * row = col->left;
    while(row!=col){
      row->column->size+=1;
      row->up->down=row;
      row->down->up=row;
      row=row->left;
      }
    col=col->up;
    }
  column->left->right=column;
  column->right->left=column;
  }

void dlx::cover(dlxcolumn * column){
  column->left->right=column->right;
  column->right->left=column->left;
  dlxnode * col = column->down;
  while(col!=column){
    dlxnode * row = col->right;
    while(row!=col){
      row->up->down=row->down;
      row->down->up=row->up;
      row->column->size-=1;
      row=row->right;
      }
    col=col->down;
    }
  }

dlxnode * dlx::initrow(int * data,int size){
  FXASSERT(size>0);
  dlxnode * prev=NULL;
  dlxnode * first=NULL;
  for (int i=0;i<size;i++){
    dlxcolumn * node = (dlxcolumn*)root->right;
    dlxnode * newnode=NULL;
    while(node!=root){
      if (data[i]==node->name) {
        newnode=&nodes[nindex++];
        node->up->down=newnode;
        newnode->down=node;
        newnode->up=node->up;
        node->up=newnode;
        node->size+=1;
        newnode->column=node;
        newnode->left=prev;
        if (prev)
          prev->right=newnode;
        else
          first=newnode;

        prev=newnode;
        break;
        }
      node=(dlxcolumn*)node->right;
      }
    FXASSERT(newnode);
    }
  FXASSERT(first);
  first->left=prev;
  prev->right=first;
  return first;
  }


void dlx::removeRow(dlxnode * row){
  dlxnode * node=row->right;
  while(node!=row) {
    cover(node->column);
    node=node->right;
    }
  }

void dlx::insertRow(dlxnode * row){
  dlxnode * node = row->left;
  while(node!=row) {
    uncover(node->column);
    node=node->left;
    }
  }

dlxcolumn * dlx::nextColumn() {
  int minsize=INT_MAX;
  dlxcolumn * c = (dlxcolumn*)root->right;
  dlxcolumn * column = c;
  while(c!=root) {
    if (c->size<minsize) {
      column=c;
      minsize=c->size;
      }
    c=(dlxcolumn*)c->right;
    }
  return column;
  }

bool dlx::findFirst(int n) {
  if (root->right==root) {
    return true;
    }
  else {
    dlxcolumn * c = nextColumn();
    cover(c);
    dlxnode * r = c->down;
    while(r!=c) {

      removeRow(r);

      solution[n]=r;

      if (findFirst(n+1)) return true;

      insertRow(r);

      r=r->down;
      }
    uncover(c);
    }
  return false;
  }


bool dlx::findAll(int n) {
  if (root->right==root) {
    return true;
    }
  else {
    dlxcolumn * c = nextColumn();
    cover(c);
    dlxnode * r = c->down;
    while(r!=c) {

      removeRow(r);

      solution[n]=r;

      findFirst(n+1);

      insertRow(r);

      r=r->down;
      }
    uncover(c);
    }
  return false;
  }








